import 'dart:async';
import 'dart:js';
import 'package:nodejs_interop/node.dart';
import 'package:test/test.dart';

/// Tests the features of the [Resolver] class.
void main() => group('Resolver', () {
  loadLibrary<DnsModule>('dns');
  final NetModule net = require('net');

  group('.getServers()/.setServers()', () {
    test('should return a non-empty list', () {
      expect(new Resolver().getServers(), isNotEmpty);
    });

    test('should get and set the list of DNS servers', () {
      var servers = const ['127.0.0.1', '::1'];
      var resolver = new Resolver()..setServers(servers);
      expect(resolver.getServers(), orderedEquals(servers));
    });
  });

  group('.resolve4()', () {
    test('should resolve the hostname into a list of IPv4 addresses', () async {
      var completer = new Completer<List>();
      new Resolver().resolve4('belin.io', allowInterop((error, [addresses]) {
        if (error == null) completer.complete(addresses);
        else completer.completeError(error);
      }));

      var addresses = await completer.future;
      expect(addresses, hasLength(1));
      expect(net.isIPv4(addresses.first), isTrue);
    });

    test('should resolve the hostname into a list of IPv4 `AddressRecord` instances', () async {
      var completer = new Completer<List>();
      new Resolver().resolve4('belin.io', new ResolveOptions(ttl: true), allowInterop((error, [addresses]) {
        if (error == null) completer.complete(addresses);
        else completer.completeError(error);
      }));

      var addresses = await completer.future;
      expect(addresses, hasLength(1));
      expect(net.isIPv4(addresses.first.address), isTrue);
    });
  });

  group('.resolve6()', () {
    test('should resolve the hostname into a list of IPv6 addresses', () async {
      var completer = new Completer<List>();
      new Resolver().resolve6('google.com', allowInterop((error, [addresses]) {
        if (error == null) completer.complete(addresses);
        else completer.completeError(error);
      }));

      var addresses = await completer.future;
      expect(addresses, hasLength(1));
      expect(net.isIPv6(addresses.first), isTrue);
    });

    test('should resolve the hostname into a list of IPv6 `AddressRecord` instances', () async {
      var completer = new Completer<List>();
      new Resolver().resolve6('google.com', new ResolveOptions(ttl: true), allowInterop((error, [addresses]) {
        if (error == null) completer.complete(addresses);
        else completer.completeError(error);
      }));

      var addresses = await completer.future;
      expect(addresses, hasLength(1));
      expect(net.isIPv6(addresses.first.address), isTrue);
    });
  });

  group('.resolveAny()', () {
    test('should resolve the hostname into a list of `DnsRecord` instances', () async {
      var completer = new Completer<List<DnsRecord>>();
      new Resolver().resolveAny('belin.io', allowInterop((error, [records]) {
        if (error == null) completer.complete(records);
        else completer.completeError(error);
      }));

      expect(await completer.future, isNotEmpty);
    });
  });

  group('.resolveCname()', () {
    test('should resolve the hostname into a list of canonical names', () async {
      var completer = new Completer<List<String>>();
      new Resolver().resolveCname('www.belin.io', allowInterop((error, [records]) {
        if (error == null) completer.complete(records);
        else completer.completeError(error);
      }));

      var records = await completer.future;
      expect(records, orderedEquals(<String>['ns.belin.io']));
    });
  });

  group('.resolveMx()', () {
    test('should resolve the hostname into a list of mail exchange records', () async {
      var completer = new Completer<List<MXRecord>>();
      new Resolver().resolveMx('google.com', allowInterop((error, [records]) {
        if (error == null) completer.complete(records);
        else completer.completeError(error);
      }));

      expect(await completer.future, isNotEmpty);
    });
  });

  group('.resolveNs()', () {
    test('should resolve the hostname into a list of name servers', () async {
      var completer = new Completer<List<String>>();
      new Resolver().resolveNs('belin.io', allowInterop((error, [records]) {
        if (error == null) completer.complete(records);
        else completer.completeError(error);
      }));

      var records = await completer.future;
      expect(records, unorderedEquals(<String>['ns-39-c.gandi.net', 'ns-68-b.gandi.net', 'ns-230-a.gandi.net']));
    });
  });

  group('.resolveSoa()', () {
    test('should resolve the hostname into a start of authority record', () async {
      var completer = new Completer<SoaRecord>();
      new Resolver().resolveSoa('belin.io', allowInterop((error, [record]) {
        if (error == null) completer.complete(record);
        else completer.completeError(error);
      }));

      var record = await completer.future;
      expect(record.hostmaster, equals('hostmaster.gandi.net'));
      expect(record.nsname, equals('ns1.gandi.net'));
    });
  });

  group('.resolveTxt()', () {
    test('should resolve the hostname into a list of text queries', () async {
      var completer = new Completer<List<List<String>>>();
      new Resolver().resolveTxt('belin.io', allowInterop((error, [records]) {
        if (error == null) completer.complete(records);
        else completer.completeError(error);
      }));

      var entries = <String>[];
      var records = await completer.future;
      for (var record in records) record.forEach(entries.add);
      expect(entries, unorderedEquals(<String>['firebase=cb-belin', 'google-site-verification=4Lpj0fzF1aePRZ_Zj3SOSn0Gv24ztXPxZ_lkTP1ouFQ']));
    });
  });

  group('.reverse()', () {
    test('should resolve the address into a list of hostnames', () async {
      var completer = new Completer<List<String>>();
      new Resolver().reverse('192.30.253.113', allowInterop((error, [hostnames]) {
        if (error == null) completer.complete(hostnames);
        else completer.completeError(error);
      }));

      var hostnames = await completer.future;
      expect(hostnames, orderedEquals(<String>['lb-192-30-253-113-iad.github.com']));
    });
  });
});
